from functools import wraps
from typing import Any, Callable, Dict, Tuple

from .exceptions import SkipTest

Decorator = Callable[[Callable[[], None]], Callable[[Tuple[Any], Dict[str, Any]], None]]

def skip_test(reason: str | None=None) -> Decorator:
    """
    Calling this during a test method or set_up() skips the current test.
    """

    def decorator(func: Callable[[], None]):
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any):
            raise SkipTest(reason)
        return wrapper
    return decorator


def skip_test_if(condition: bool, reason: str | None=None) -> Decorator:
    """
    Calling this during a test method or set_up() skips the current test 
    if the condition evaluates to True.
    """

    def decorator(func: Callable[[], None]):
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any):
            if condition:
                raise SkipTest(reason)
            else:
                func(*args, **kwargs)
        return wrapper
    return decorator


def skip_test_unless(condition: bool, reason: str | None=None) -> Decorator:
    """
    Calling this during a test method or set_up() skips the current test 
    unless the condition evaluates to True.
    """

    def decorator(func: Callable[[], None]):
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any):
            if not condition:
                raise SkipTest(reason)
            else:
                func(*args, **kwargs)
        return wrapper
    return decorator



from functools import wraps
from typing import Any, Callable

from .exceptions import SkipTest
from .skip_test import Decorator


def output_msg(type: str, msg: str) -> None:
    """
    Prints a message with a green tick if the type is 'suc'; otherwise,
    prints it with a red cross.
    """

    if type == 'err':
        print(f'\033[31m\u00d7 {msg}\033[0m')
    elif type =='suc':
        print(f'\033[32m\u2713 {msg}\033[0m')


def show_message(fail: str | None=None, success: str | None=None) -> Decorator:
    """
    Decorator used to print a failure message if a test fails or a success
    message if a test passes.

    If one or the other is specified, show_message will only print for 
    that case. For example, if you don't want to print anything when a 
    test passes, you can omit the success message.
    """
    
    def decorator(func: Callable[[], None]):
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any):
            try:
                func(*args, **kwargs)

                if success:
                    output_msg('suc', success)

            except (AssertionError, SkipTest) as err:
                fail_msg = fail

                first = err.args[0]['first']
                if fail_msg and '%f' in fail_msg:
                    fail_msg = fail_msg.replace('%f', str(first))

                second = err.args[0]['second']
                if fail_msg and '%s' in fail_msg:
                    fail_msg = fail_msg.replace('%s', str(second))

                if fail_msg:
                    output_msg('err', fail_msg)

                raise err

        return wrapper
    return decorator



__all__ = [
    'SkipTest',
    'show_message',
    'skip_test',
    'skip_test_if',
    'skip_test_unless',
    'TestCase',
]

from .exceptions import SkipTest
from .message import show_message
from .skip_test import skip_test, skip_test_if, skip_test_unless
from .testcase import TestCase



from sys import argv
from typing import List
import subprocess
import os


def main() -> None:
    """
    This is the main function of the program. It runs tests inside a
    specified directory or executes the test file itself if provided.

    Usage: python -m colorful-test <directory_name>/<file_name>
    """

    args = argv[1:]

    if len(args) < 2:
        print('Usage: python -m colorful-test <directory_name>/<file_name>')


def execute_file(filename: str) -> None:
    """Uses subprocess to run a test file"""
    subprocess.run(['python3', filename], capture_output=True, text=True)


def run_tests(args: List[str]) -> None:
    """
    Takes a list of tests and runs them.
    """

    for arg in args:
        if os.path.isdir(arg):
            run_tests(os.listdir(arg))

        elif arg.startswith('test') and arg.endswith('.py'):
            execute_file(arg)


if __name__ == '__main__':
    main()
